@@ -2,12 +2,12 @@
 # @Author: Kelvin
 # @Date:   2020-05-18 00:15:00
 # @Last Modified by:   Kelvin
-# @Last Modified time: 2022-06-18 15:05:57
+# @Last Modified time: 2022-07-07 12:40:40
 """plotting module."""
+import matplotlib.patches as mpatches
 import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
-import random
 import seaborn as sns
 
 from anndata import AnnData
@@ -15,16 +15,16 @@ from itertools import combinations, cycle
 from matplotlib.axes import Axes
 from matplotlib.figure import Figure
 from plotnine import (
-    ggplot,
-    theme_classic,
     aes,
     geom_line,
-    xlab,
-    ylab,
-    options,
+    ggplot,
     ggtitle,
     labs,
+    options,
     scale_color_manual,
+    theme_classic,
+    xlab,
+    ylab,
 )
 from scanpy.plotting import palettes
 from scanpy.plotting._tools.scatterplots import embedding
@@ -44,7 +44,24 @@ def clone_rarefaction(
     color: str,
     clone_key: Optional[str] = None,
     palette: Optional[Sequence] = None,
-    figsize: Tuple[Union[int, float], Union[int, float]] = (6, 4),
+    figsize: Tuple[Union[int, float], Union[int, float]] = (5, 3),
+    chain_status_include: List[
+        Literal[
+            "Single pair",
+            "Orphan VDJ",
+            "Orphan VDJ-exception",
+            "Orphan VJ",
+            "Orphan VJ-exception",
+            "Extra pair",
+            "Extra pair-exception",
+        ]
+    ] = [
+        "Single pair",
+        "Orphan VDJ",
+        "Orphan VDJ-exception",
+        "Extra pair",
+        "Extra pair-exception",
+    ],
     save: Optional[str] = None,
 ) -> ggplot:
     """
@@ -62,6 +79,11 @@ def clone_rarefaction(
         Color mapping for unique elements in color. Will try to retrieve from AnnData `.uns` slot if present.
     figsize :  Tuple[Union[int,float], Union[int,float]]
         Size of plot.
+    chain_status_exclude : List
+        Non-exhaustive list of chains to into the analysis. e.g.
+        "Single pair", "Orphan VDJ", "Orphan VDJ-exception",
+        "Orphan VJ", "Orphan VJ-exception", "Extra pair",
+        "Extra pair-exception",
     save : str, Optional
         Save path.
 
@@ -69,9 +91,9 @@ def clone_rarefaction(
     -------
     rarefaction curve plot.
     """
-    if self.__class__ == AnnData:
+    if isinstance(self, AnnData):
         metadata = self.obs.copy()
-    elif self.__class__ == Dandelion:
+    elif isinstance(self, Dandelion):
         metadata = self.metadata.copy()
     if clone_key is None:
         clonekey = "clone_id"
@@ -79,7 +101,11 @@ def clone_rarefaction(
         clonekey = clone_key
 
     groups = list(set(metadata[color]))
-    metadata = metadata[metadata["contig_QC_pass"].isin([True, "True"])]
+    if "contig_QC_pass" in metadata:
+        metadata = metadata[metadata["contig_QC_pass"].isin(TRUES)]
+    elif "chain_status" in metadata:
+        metadata = metadata[metadata["chain_status"].isin(chain_status_include)]
+
     if type(metadata[clonekey]) == "category":
         metadata[clonekey] = metadata[clonekey].cat.remove_unused_categories()
     res = {}
@@ -105,7 +131,11 @@ def clone_rarefaction(
 
     # append the results to a dictionary
     rarecurve = {}
-    for i in tqdm(range(0, nr), desc="Calculating rarefaction curve "):
+    for i in tqdm(
+        range(0, nr),
+        desc="Calculating rarefaction curve ",
+        bar_format="{l_bar}{bar:10}{r_bar}{bar:-10b}",
+    ):
         n = np.arange(1, tot[i], step=10)
         if n[-1:] != tot[i]:
             n = np.append(n, tot[i])
@@ -132,7 +162,7 @@ def clone_rarefaction(
 
     options.figure_size = figsize
     if palette is None:
-        if self.__class__ == AnnData:
+        if isinstance(self, AnnData):
             try:
                 pal = self.uns[str(color) + "_colors"]
             except:
@@ -219,16 +249,6 @@ def clone_rarefaction(
     return p
 
 
-def random_palette(n: int) -> Sequence:
-    """A random palette of colours."""
-    # a list of 900+colours
-    cols = list(sns.xkcd_rgb.keys())
-    # if max_colors_needed1 > len(cols):
-    cols2 = list(sns.color_palette("husl", n))
-    palette = random.sample(sns.xkcd_palette(cols) + cols2, n)
-    return palette
-
-
 def clone_network(
     adata: AnnData, basis: str = "vdj", edges: bool = True, **kwargs
 ) -> Union[Figure, Axes, None]:
@@ -256,7 +276,7 @@ def barplot(
     self: Union[AnnData, Dandelion],
     color: str,
     palette: str = "Set1",
-    figsize: Tuple[Union[int, float], Union[int, float]] = (12, 4),
+    figsize: Tuple[Union[int, float], Union[int, float]] = (8, 3),
     normalize: bool = True,
     sort_descending: bool = True,
     title: Optional[str] = None,
@@ -281,7 +301,7 @@ def barplot(
         or a dictionary mapping hue levels to matplotlib colors.
         See [seaborn.barplot](https://seaborn.pydata.org/generated/seaborn.barplot.html).
     figsize : Tuple[Union[int,float], Union[int,float]]
-        figure size. Default is (12, 4).
+        figure size. Default is (8, 3).
     normalize : bool
         if True, will return as proportion out of 1.
         Otherwise False will return counts. Default is True.
@@ -302,9 +322,9 @@ def barplot(
     -------
     a seaborn barplot.
     """
-    if self.__class__ == Dandelion:
+    if isinstance(self, Dandelion):
         data = self.metadata.copy()
-    elif self.__class__ == AnnData:
+    elif isinstance(self, AnnData):
         data = self.obs.copy()
 
     if min_clone_size is None:
@@ -353,7 +373,7 @@ def stackedbarplot(
     self: Union[AnnData, Dandelion],
     color: str,
     groupby: Optional[str],
-    figsize: Tuple[Union[int, float], Union[int, float]] = (12, 4),
+    figsize: Tuple[Union[int, float], Union[int, float]] = (8, 3),
     normalize: bool = False,
     title: Optional[str] = None,
     sort_descending: bool = True,
@@ -377,7 +397,7 @@ def stackedbarplot(
     groupby : str
         column name in metadata to split by during plotting.
     figsize : Tuple[Union[int,float], Union[int,float]]
-        figure size. Default is (12, 4).
+        figure size. Default is (8, 3).
     normalize : bool
         if True, will return as proportion out of 1, otherwise False will return counts. Default is True.
     sort_descending : bool
@@ -403,9 +423,9 @@ def stackedbarplot(
     -------
     stacked bar plot.
     """
-    if self.__class__ == Dandelion:
+    if isinstance(self, Dandelion):
         data = self.metadata.copy()
-    elif self.__class__ == AnnData:
+    elif isinstance(self, AnnData):
         data = self.obs.copy()
     # quick fix to prevent dropping of nan
     data[groupby] = [str(l) for l in data[groupby]]
@@ -444,7 +464,7 @@ def stackedbarplot(
     def _plot_bar_stacked(
         dfall: pd.DataFrame,
         labels: Optional[Sequence] = None,
-        figsize: Tuple[Union[int, float], Union[int, float]] = (12, 4),
+        figsize: Tuple[Union[int, float], Union[int, float]] = (8, 3),
         title: str = "multiple stacked bar plot",
         xtick_rotation: Optional[Union[int, float]] = None,
         legend_options: Tuple[str, Tuple[float, float], int] = None,
@@ -561,7 +581,7 @@ def spectratype(
     color: str,
     groupby: str,
     locus: str,
-    figsize: Tuple[Union[int, float], Union[int, float]] = (6, 4),
+    figsize: Tuple[Union[int, float], Union[int, float]] = (5, 3),
     width: Optional[Union[int, float]] = None,
     title: Optional[str] = None,
     xtick_rotation: Optional[Union[int, float]] = None,
@@ -584,7 +604,7 @@ def spectratype(
     locus : str
         either IGH or IGL.
     figsize : Tuple[Union[int,float], Union[int,float]]
-        figure size. Default is (6, 4).
+        figure size. Default is (5, 3).
     width : float, int, Optional
         width of bars.
     title : str, Optional
@@ -606,19 +626,15 @@ def spectratype(
     -------
     spectratype plot
     """
-    if self.__class__ == Dandelion:
+    if isinstance(self, Dandelion):
         data = self.data.copy()
+        if "ambiguous" in self.data:
+            data = data[data["ambiguous"] == "F"].copy()
     else:
-        try:
-            data = self.copy()
-        except:
-            AttributeError(
-                "Please provide a <class 'Dandelion'> class object or a pandas dataframe instead of %s."
-                % self.__class__
-            )
-
-    if "locus" not in data.columns:
-        raise AttributeError("Please ensure dataframe contains 'locus' column")
+        raise ValueError(
+            "Please provide a <class 'Dandelion'> class object instead of %s."
+            % type(self)
+        )
 
     if type(locus) is not list:
         locus = [locus]
@@ -641,7 +657,7 @@ def spectratype(
     def _plot_spectra_stacked(
         dfall: pd.DataFrame,
         labels: Optional[Sequence] = None,
-        figsize: Tuple[Union[int, float], Union[int, float]] = (6, 4),
+        figsize: Tuple[Union[int, float], Union[int, float]] = (5, 3),
         title: str = "multiple stacked bar plot",
         width: Optional[Union[int, float]] = None,
         xtick_rotation: Optional[Union[int, float]] = None,
@@ -742,7 +758,7 @@ def spectratype(
 
 
 def clone_overlap(
-    self: Union[AnnData, Dandelion],
+    self: AnnData,
     groupby: str,
     colorby: str,
     min_clone_size: Optional[int] = None,
@@ -769,12 +785,12 @@ def clone_overlap(
 
     Parameters
     ----------
-    self : Dandelion, AnnData
-        `Dandelion` or `AnnData` object.
+    self : AnnData
+        `AnnData` object.
     groupby : str
-        column name in obs/metadata for collapsing to nodes in circos plot.
+        column name in obs for collapsing to nodes in circos plot.
     colorby : str
-        column name in obs/metadata for grouping and color of nodes in circos plot.
+        column name in obs for grouping and color of nodes in circos plot.
     min_clone_size : int, Optional
         minimum size of clone for plotting connections. Defaults to 2 if left as None.
     weighted_overlapt : bool
@@ -827,111 +843,17 @@ def clone_overlap(
     else:
         clone_ = clone_key
 
-    if self.__class__ == AnnData:
+    if isinstance(self, AnnData):
         data = self.obs.copy()
         # get rid of problematic rows that appear because of category conversion?
         if "clone_overlap" in self.uns:
             overlap = self.uns["clone_overlap"].copy()
         else:
-            allgroups = list(data[groupby].unique())
-            data = data[
-                ~(
-                    data[clone_].isin(
-                        [
-                            np.nan,
-                            "nan",
-                            "NaN",
-                            "No_contig",
-                            "unassigned",
-                            "None",
-                            None,
-                        ]
-                    )
-                )
-            ]
-            # prepare a summary table
-            datc_ = data[clone_].str.split("|", expand=True).stack()
-            datc_ = pd.DataFrame(datc_)
-            datc_.reset_index(drop=False, inplace=True)
-            datc_.columns = ["cell_id", "tmp", clone_]
-            datc_.drop("tmp", inplace=True, axis=1)
-            datc_ = datc_[
-                ~(
-                    datc_[clone_].isin(
-                        [
-                            "",
-                            np.nan,
-                            "nan",
-                            "NaN",
-                            "No_contig",
-                            "unassigned",
-                            None,
-                        ]
-                    )
-                )
-            ]
-            dictg_ = dict(data[groupby])
-            datc_[groupby] = [dictg_[l] for l in datc_["cell_id"]]
-
-            overlap = pd.crosstab(datc_[clone_], datc_[groupby])
-            for x in allgroups:
-                if x not in overlap:
-                    overlap[x] = 0
-            if min_size == 0:
-                raise ValueError("min_size must be greater than 0.")
-            if not weighted_overlap:
-                if min_size > 2:
-                    overlap[overlap < min_size] = 0
-                    overlap[overlap >= min_size] = 1
-                elif min_size == 2:
-                    overlap[overlap >= min_size] = 1
-
-            overlap.index.name = None
-            overlap.columns.name = None
-    elif self.__class__ == Dandelion:
-        data = self.metadata.copy()
-        allgroups = list(data[groupby].unique())
-        # get rid of problematic rows that appear because of category conversion?
-        data = data[
-            ~(
-                data[clone_].isin(
-                    [
-                        np.nan,
-                        "nan",
-                        "NaN",
-                        "No_contig",
-                        "unassigned",
-                        "None",
-                        None,
-                    ]
-                )
+            raise KeyError(
+                "`clone_overlap` not found in `adata.uns`. Did you run `tl.clone_overlap`?"
             )
-        ]
-
-        # prepare a summary table
-        datc_ = data[clone_].str.split("|", expand=True).stack()
-        datc_ = pd.DataFrame(datc_)
-        datc_.reset_index(drop=False, inplace=True)
-        datc_.columns = ["cell_id", "tmp", clone_]
-        datc_.drop("tmp", inplace=True, axis=1)
-        dictg_ = dict(data[groupby])
-        datc_[groupby] = [dictg_[l] for l in datc_["cell_id"]]
-
-        overlap = pd.crosstab(datc_[clone_], datc_[groupby])
-        for x in allgroups:
-            if x not in overlap:
-                overlap[x] = 0
-        if min_size == 0:
-            raise ValueError("min_size must be greater than 0.")
-        if not weighted_overlap:
-            if min_size > 2:
-                overlap[overlap < min_size] = 0
-                overlap[overlap >= min_size] = 1
-            elif min_size == 2:
-                overlap[overlap >= min_size] = 1
-
-        overlap.index.name = None
-        overlap.columns.name = None
+    else:
+        raise ValueError("Please provide a AnnData object.")
 
     edges = {}
     if not weighted_overlap:
@@ -1011,7 +933,7 @@ def clone_overlap(
         weighted_attr = "weight"
 
     if color_mapping is None:
-        if self.__class__ == AnnData:
+        if isinstance(self, AnnData):
             if str(colorby) + "_colors" in self.uns:
                 if pd.api.types.is_categorical_dtype(self.obs[groupby]):
                     colorby_dict = dict(
@@ -1111,3 +1033,60 @@ def clone_overlap(
             plt.savefig(save, bbox_inches="tight", **kwargs)
         if return_graph:
             return G
+
+
+def productive_ratio(
+    adata: AnnData,
+    figsize: Tuple[Union[int, float], Union[int, float]] = (8, 4),
+    palette: List = ["lightblue", "darkblue"],
+    fontsize: Union[int, float] = 8,
+    rotation: Union[int, float] = 90,
+    legend_kwargs: Dict = {
+        "bbox_to_anchor": (1, 0.5),
+        "loc": "center left",
+        "frameon": False,
+    },
+):
+    """Plot productive/non-productive contig ratio from AnnData (cell level).
+
+    Parameters
+    ----------
+    adata : AnnData
+        AnnData object with `.uns['productive_ratio']` computed from
+        `tl.productive_ratio`.
+    figsize : Tuple[Union[int, float], Union[int, float]], optional
+        Size of figure.
+    palette : List[str, str], optional
+        List of colours to plot non-productive and productive respectively.
+    fontsize : Union[int, float], optional
+        Font size of x and y tick labels.
+    rotation : Union[int, float], optional
+        Rotation of x tick labels.
+    legend_kwargs : Dict, optional
+        Any additional kwargs to `plt.legend`
+    """
+    res = adata.uns["productive_ratio"]["results"]
+    locus = adata.uns["productive_ratio"]["locus"]
+    groupby = adata.uns["productive_ratio"]["groupby"]
+
+    plt.figure(figsize=figsize)
+    ax = sns.barplot(
+        x=groupby, y="productive+non-productive", data=res, color=palette[0]
+    )
+    ax = sns.barplot(
+        x=groupby, y="productive", data=res, color=palette[1], ax=ax
+    )
+    legend = [
+        mpatches.Patch(
+            color=palette[0], label="% with non-productive " + locus
+        ),
+        mpatches.Patch(color=palette[1], label="% with productive " + locus),
+    ]
+    plt.xticks(fontsize=fontsize, rotation=rotation)
+    plt.yticks(fontsize=fontsize)
+    plt.xlabel("")
+    plt.ylabel("")
+    ax.set(ylim=(0, 100))
+    plt.title(locus)
+    # add legend
+    plt.legend(handles=legend, **legend_kwargs)
