@@ -2,34 +2,36 @@
 # @Author: kt16
 # @Date:   2020-05-12 18:42:02
 # @Last Modified by:   Kelvin
-# @Last Modified time: 2022-06-18 14:30:56
+# @Last Modified time: 2022-07-03 21:52:05
 """init module."""
 from . import external
 from ._preprocessing import (
-    format_fasta,
-    format_fastas,
     assign_isotype,
     assign_isotypes,
-    reassign_alleles,
-    reannotate_genes,
+    calculate_threshold,
+    check_contigs,
     create_germlines,
     filter_contigs,
+    format_fasta,
+    format_fastas,
     quantify_mutations,
-    calculate_threshold,
+    reannotate_genes,
+    reassign_alleles,
 )
 from .external._preprocessing import recipe_scanpy_qc
 
 __all__ = [
-    "external",
-    "format_fasta",
-    "format_fastas",
     "assign_isotype",
     "assign_isotypes",
-    "reassign_alleles",
-    "reannotate_genes",
-    "create_germlines",
-    "filter_contigs",
-    "quantify_mutations",
     "calculate_threshold",
+    "check_contigs",
+    "create_germlines",
+    "external",
+    "filter_contigs",
+    "format_fasta",
+    "format_fastas",
+    "quantify_mutations",
+    "reannotate_genes",
+    "reassign_alleles",
     "recipe_scanpy_qc",
 ]
