@@ -2,23 +2,24 @@
 # @Author: kt16
 # @Date:   2020-05-12 14:01:32
 # @Last Modified by:   Kelvin
-# @Last Modified time: 2022-06-18 14:41:28
+# @Last Modified time: 2022-07-06 08:08:00
 """io module."""
-
-import _pickle as cPickle
 import bz2
 import gzip
 import json
+import os
+import re
+
+import _pickle as cPickle
 import networkx as nx
 import numpy as np
-import os
 import pandas as pd
-import re
 
 from anndata import AnnData
 from collections import defaultdict, OrderedDict
 from os import PathLike
-from typing import Union, Sequence, Optional
+from scanpy import logging as logg
+from typing import Union, Sequence, Optional, List
 
 from ..utilities._core import *
 from ..utilities._utilities import *
@@ -192,7 +193,95 @@ def read_h5(filename: str = "dandelion_data.h5") -> Dandelion:
         pass
 
     try:
-        edges = pd.read_hdf(filename, "edges")
+        g_0 = pd.read_hdf(filename, "graph/graph_0")
+        g_1 = pd.read_hdf(filename, "graph/graph_1")
+        g_0 = g_0 + 1
+        g_0 = g_0.fillna(0)
+        g_1 = g_1 + 1
+        g_1 = g_1.fillna(0)
+        graph0 = nx.from_pandas_adjacency(g_0)
+        graph1 = nx.from_pandas_adjacency(g_1)
+        for u, v, d in graph0.edges(data=True):
+            d["weight"] = d["weight"] - 1
+        for u, v, d in graph1.edges(data=True):
+            d["weight"] = d["weight"] - 1
+        graph = (graph0, graph1)
+    except:
+        pass
+
+    with h5py.File(filename, "r") as hf:
+        try:
+            layout0 = {}
+            for k in hf["layout/layout_0"].attrs.keys():
+                layout0.update({k: np.array(hf["layout/layout_0"].attrs[k])})
+            layout1 = {}
+            for k in hf["layout/layout_1"].attrs.keys():
+                layout1.update({k: np.array(hf["layout/layout_1"].attrs[k])})
+            layout = (layout0, layout1)
+        except:
+            pass
+
+        germline = {}
+        try:
+            for g in hf["germline"].attrs:
+                germline.update({g: hf["germline"].attrs[g]})
+        except:
+            pass
+
+        try:
+            threshold = float(np.array(hf["threshold"]))
+        except:
+            threshold = None
+
+    constructor = {}
+    constructor["data"] = data
+    if "metadata" in locals():
+        constructor["metadata"] = metadata
+    if "germline" in locals():
+        constructor["germline"] = germline
+    if "layout" in locals():
+        constructor["layout"] = layout
+    if "graph" in locals():
+        constructor["graph"] = graph
+    try:
+        res = Dandelion(**constructor)
+    except:
+        res = Dandelion(**constructor, initialize=False)
+
+    if "threshold" in locals():
+        res.threshold = threshold
+    else:
+        pass
+    return res
+
+
+def read_h5ddl(filename: str = "dandelion_data.h5ddl") -> Dandelion:
+    """
+    Read in and returns a `Dandelion` class from .h5 format.
+
+    Parameters
+    ----------
+    filename : str
+        path to `.h5` file
+
+    Returns
+    -------
+    `Dandelion` object.
+    """
+    try:
+        data = pd.read_hdf(filename, "data")
+        # data = sanitize_data(data)
+
+        # if check_mix_dtype(data):
+        #     for x in return_mix_dtype(data):
+        #        data[x].replace('', pd.NA, inplace=True)
+        #     data = sanitize_data(data)
+    except:
+        raise AttributeError(
+            "{} does not contain attribute `data`".format(filename)
+        )
+    try:
+        metadata = pd.read_hdf(filename, "metadata")
     except:
         pass
 
@@ -243,8 +332,6 @@ def read_h5(filename: str = "dandelion_data.h5") -> Dandelion:
         constructor["metadata"] = metadata
     if "germline" in locals():
         constructor["germline"] = germline
-    if "edges" in locals():
-        constructor["edges"] = edges
     if "layout" in locals():
         constructor["layout"] = layout
     if "graph" in locals():
@@ -373,7 +460,11 @@ def from_scirpy(adata: AnnData) -> Dandelion:
 
 
 def concat(
-    arrays: Sequence[Union[pd.DataFrame, Dandelion]], check_unique: bool = True
+    arrays: Sequence[Union[pd.DataFrame, Dandelion]],
+    check_unique: bool = True,
+    sep: str = "-",
+    suffixes: Optional[List] = None,
+    prefixes: Optional[List] = None,
 ) -> Dandelion:
     """
     Concatenate dataframe and return as `Dandelion` object.
@@ -385,6 +476,14 @@ def concat(
     check_unique : bool
         Check the new index for duplicates. Otherwise defer the check until necessary.
         Setting to False will improve the performance of this method.
+    sep : str
+        the separator to append suffix/prefix.
+    suffixes : Optional, List
+        List of suffixes to append to sequence_id.
+        If both suffixes and prefixes are None and check_unique is True, then a sequential number suffix will be appended.
+    prefixes : Optional, List
+        List of prefixes to append to sequence_id.
+        If both suffixes and prefixes are None and check_unique is True, then a sequential number suffix will be appended.
 
     Returns
     -------
@@ -397,14 +496,41 @@ def concat(
     except:
         arrays_ = [x.copy() for x in arrays]
 
+    if (suffixes is not None) and (prefixes is not None):
+        raise ValueError("Please provide only prefixes or suffixes, not both.")
+
+    if suffixes is not None:
+        if len(arrays_) != len(suffixes):
+            raise ValueError(
+                "Please provide the same number of suffixes as the number of objects to concatenate."
+            )
+
+    if prefixes is not None:
+        if len(arrays_) != len(prefixes):
+            raise ValueError(
+                "Please provide the same number of prefixes as the number of objects to concatenate."
+            )
+
     if check_unique:
         try:
             df = pd.concat(arrays_, verify_integrity=True)
         except:
             for i in range(0, len(arrays)):
-                arrays_[i]["sequence_id"] = [
-                    x + "__" + str(i) for x in arrays_[i]["sequence_id"]
-                ]
+                if (suffixes is None) and (prefixes is None):
+                    ii = str(i)
+                    arrays_[i]["sequence_id"] = [
+                        x + sep + ii for x in arrays_[i]["sequence_id"]
+                    ]
+                elif suffixes is not None:
+                    ii = str(suffixes[i])
+                    arrays_[i]["sequence_id"] = [
+                        x + sep + ii for x in arrays_[i]["sequence_id"]
+                    ]
+                elif prefixes is not None:
+                    ii = str(prefixes[i])
+                    arrays_[i]["sequence_id"] = [
+                        ii + sep + x for x in arrays_[i]["sequence_id"]
+                    ]
             arrays_ = [load_data(x) for x in arrays_]
             df = pd.concat(arrays_, verify_integrity=True)
     else:
@@ -466,8 +592,7 @@ def read_10x_vdj(
         json_idx = [i for i, j in enumerate(filelist) if j.endswith(".json")]
         if len(csv_idx) == 1:
             file = str(path) + "/" + str(filelist[csv_idx[0]])
-            if verbose:
-                print("Reading {}".format(str(file)))
+            logg.info("Reading {}".format(str(file)))
             raw = pd.read_csv(str(file))
             raw.set_index("contig_id", drop=False, inplace=True)
             fasta_file = str(file).split("_annotations.csv")[0] + ".fasta"
@@ -477,24 +602,22 @@ def read_10x_vdj(
                 str(file).split(".csv")[0] + ".json",
             )
             if os.path.exists(json_file):
-                if verbose:
-                    print(
-                        "Found {} file. Extracting extra information.".format(
-                            str(json_file)
-                        )
+                logg.info(
+                    "Found {} file. Extracting extra information.".format(
+                        str(json_file)
                     )
+                )
                 out = parse_annotation(raw)
                 with open(json_file) as f:
                     raw_json = json.load(f)
                 out_json = parse_json(raw_json)
                 out.update(out_json)
             elif os.path.exists(fasta_file):
-                if verbose:
-                    print(
-                        "Found {} file. Extracting extra information.".format(
-                            str(fasta_file)
-                        )
+                logg.info(
+                    "Found {} file. Extracting extra information.".format(
+                        str(fasta_file)
                     )
+                )
                 seqs = {}
                 fh = open(fasta_file, "r")
                 for header, sequence in fasta_iterator(fh):
@@ -506,8 +629,7 @@ def read_10x_vdj(
         elif len(csv_idx) < 1:
             if len(json_idx) == 1:
                 json_file = str(path) + "/" + str(filelist[json_idx[0]])
-                if verbose:
-                    print("Reading {}".format(json_file))
+                logg.info("Reading {}".format(json_file))
                 if os.path.exists(json_file):
                     with open(json_file) as f:
                         raw = json.load(f)
@@ -527,8 +649,7 @@ def read_10x_vdj(
     elif os.path.isfile(str(path)):
         file = path
         if str(file).endswith(".csv"):
-            if verbose:
-                print("Reading {}.".format(str(file)))
+            logg.info("Reading {}.".format(str(file)))
             raw = pd.read_csv(str(file))
             raw.set_index("contig_id", drop=False, inplace=True)
             fasta_file = str(file).split("_annotations.csv")[0] + ".fasta"
@@ -538,24 +659,22 @@ def read_10x_vdj(
                 str(file).split(".csv")[0] + ".json",
             )
             if os.path.exists(json_file):
-                if verbose:
-                    print(
-                        "Found {} file. Extracting extra information.".format(
-                            str(json_file)
-                        )
+                logg.info(
+                    "Found {} file. Extracting extra information.".format(
+                        str(json_file)
                     )
+                )
                 out = parse_annotation(raw)
                 with open(json_file) as f:
                     raw_json = json.load(f)
                 out_json = parse_json(raw_json)
                 out.update(out_json)
             elif os.path.exists(fasta_file):
-                if verbose:
-                    print(
-                        "Found {} file. Extracting extra information.".format(
-                            str(fasta_file)
-                        )
+                logg.info(
+                    "Found {} file. Extracting extra information.".format(
+                        str(fasta_file)
                     )
+                )
                 seqs = {}
                 fh = open(fasta_file, "r")
                 for header, sequence in fasta_iterator(fh):
@@ -566,8 +685,7 @@ def read_10x_vdj(
                 out = parse_annotation(raw)
         elif str(file).endswith(".json"):
             if os.path.exists(file):
-                if verbose:
-                    print("Reading {}".format(file))
+                logg.info("Reading {}".format(file))
                 with open(file) as f:
                     raw = json.load(f)
                 out = parse_json(raw)
